diff -ur vorbis-CLEAN/configure.in vorbis/configure.in
--- vorbis-CLEAN/configure.in	Tue Aug 15 11:09:31 2000
+++ vorbis/configure.in	Sat Aug 26 06:17:30 2000
@@ -60,7 +60,8 @@
 	*86-*-linux*)
 		DEBUG="-g -Wall -D_REENTRANT -D__NO_MATH_INLINES -fsigned-char"
 		OPT="-O20 -ffast-math -D_REENTRANT -fsigned-char"
-		PROFILE="-pg -g -O20 -ffast-math -D_REENTRANT -fsigned-char"
+#		PROFILE="-Wall -W -pg -O20 -ffast-math -D_REENTRANT -fsigned-char -fno-inline -static"
+		PROFILE="-Wall -W -pg -O20 -ffast-math -mno-ieee-fp -D_REENTRANT -fsigned-char -fno-inline -static"
 		
 		# glibc < 2.1.3 has a serious FP bug in the math inline header
 		# that will cripple Vorbis.  Look to see if the magic FP stack
diff -ur vorbis-CLEAN/examples/decoder_example.c vorbis/examples/decoder_example.c
--- vorbis-CLEAN/examples/decoder_example.c	Tue Aug 15 03:05:26 2000
+++ vorbis/examples/decoder_example.c	Fri Aug 25 08:43:04 2000
@@ -239,7 +239,11 @@
 		  int16_t *ptr=convbuffer+i;
 		  double  *mono=pcm[i];
 		  for(j=0;j<bout;j++){
+#if 1
 		    int val=mono[j]*32767.;
+#else
+		    int val=mono[j]*32767.+drand48()-0.5;
+#endif
 		    /* might as well guard against clipping */
 		    if(val>32767){
 		      val=32767;
diff -ur vorbis-CLEAN/include/vorbis/codebook.h vorbis/include/vorbis/codebook.h
--- vorbis-CLEAN/include/vorbis/codebook.h	Mon Jul 17 14:55:35 2000
+++ vorbis/include/vorbis/codebook.h	Tue Aug 22 07:22:59 2000
@@ -94,6 +94,10 @@
 } encode_aux_pigeonhole;
 
 typedef struct decode_aux{
+  long   *tab;
+  int    *tabl;
+  int    tabn;
+
   long   *ptr0;
   long   *ptr1;
   long   aux;        /* number of tree entries */
diff -ur vorbis-CLEAN/lib/bitwise.c vorbis/lib/bitwise.c
--- vorbis-CLEAN/lib/bitwise.c	Fri Jul  7 03:41:43 2000
+++ vorbis/lib/bitwise.c	Thu Aug 24 21:38:16 2000
@@ -121,6 +121,22 @@
   return(m&ret);
 }
 
+/* Read in bits without advancing the bitptr; bits <= 8 */
+/* we never return 'out of bits'; we'll handle it on _adv */
+
+long _oggpack_look_huff(oggpack_buffer *b,int bits){
+  unsigned long ret;
+  unsigned long m=mask[bits];
+
+  bits+=b->endbit;
+
+  ret=b->ptr[0]>>b->endbit;
+  if(bits>8){
+    ret|=b->ptr[1]<<(8-b->endbit);  
+  }
+  return(m&ret);
+}
+
 long _oggpack_look1(oggpack_buffer *b){
   if(b->endbyte>=b->storage)return(-1);
   return((b->ptr[0]>>b->endbit)&1);
@@ -131,6 +147,16 @@
   b->ptr+=bits/8;
   b->endbyte+=bits/8;
   b->endbit=bits&7;
+}
+
+/* have to check for overflow now. return -1 on overflow */
+int _oggpack_adv_huff(oggpack_buffer *b,int bits){
+  if(b->endbyte+(b->endbit+bits-1)/8>=b->storage)return(-1);
+  bits+=b->endbit;
+  b->ptr+=bits/8;
+  b->endbyte+=bits/8;
+  b->endbit=bits&7;
+  return 0;
 }
 
 void _oggpack_adv1(oggpack_buffer *b){
diff -ur vorbis-CLEAN/lib/bitwise.h vorbis/lib/bitwise.h
--- vorbis-CLEAN/lib/bitwise.h	Fri Jul  7 03:41:43 2000
+++ vorbis/lib/bitwise.h	Tue Aug 22 04:09:40 2000
@@ -27,8 +27,10 @@
 extern void  _oggpack_readinit(oggpack_buffer *b,unsigned char *buf,int bytes);
 extern void  _oggpack_write(oggpack_buffer *b,unsigned long value,int bits);
 extern long  _oggpack_look(oggpack_buffer *b,int bits);
+extern long  _oggpack_look_huff(oggpack_buffer *b,int bits);
 extern long  _oggpack_look1(oggpack_buffer *b);
 extern void  _oggpack_adv(oggpack_buffer *b,int bits);
+extern int   _oggpack_adv_huff(oggpack_buffer *b,int bits);
 extern void  _oggpack_adv1(oggpack_buffer *b);
 extern long  _oggpack_read(oggpack_buffer *b,int bits);
 extern long  _oggpack_read1(oggpack_buffer *b);
diff -ur vorbis-CLEAN/lib/block.c vorbis/lib/block.c
--- vorbis-CLEAN/lib/block.c	Wed Jul 12 11:36:17 2000
+++ vorbis/lib/block.c	Thu Aug 24 21:37:40 2000
@@ -513,7 +513,7 @@
   return(0);
 }
 
-/* Unike in analysis, the window is only partially applied for each
+/* Unlike in analysis, the window is only partially applied for each
    block.  The time domain envelope is not yet handled at the point of
    calling (as it relies on the previous block). */
 
@@ -522,7 +522,7 @@
 
   /* Shift out any PCM that we returned previously */
   /* centerW is currently the center of the last block added */
-  if(v->pcm_returned  && v->centerW>vi->blocksizes[1]/2){
+  if(v->pcm_returned && v->centerW>vi->blocksizes[1]/2){
 
     /* don't shift too much; we need to have a minimum PCM buffer of
        1/2 long block */
@@ -589,13 +589,14 @@
 
     for(j=0;j<vi->channels;j++){
       double *pcm=v->pcm[j]+beginW;
-      
+      double *p=vb->pcm[j];
+
       /* the overlap/add section */
       for(i=beginSl;i<endSl;i++)
-	pcm[i]+=vb->pcm[j][i];
+	pcm[i]+=p[i];
       /* the remaining section */
       for(;i<sizeW;i++)
-	pcm[i]=vb->pcm[j][i];
+	pcm[i]=p[i];
     }
 
     /* track the frame number... This is for convenience, but also
diff -ur vorbis-CLEAN/lib/bookinternal.h vorbis/lib/bookinternal.h
--- vorbis-CLEAN/lib/bookinternal.h	Wed Jun 14 03:38:31 2000
+++ vorbis/lib/bookinternal.h	Thu Aug 24 21:40:19 2000
@@ -35,5 +35,7 @@
 extern long vorbis_book_decode(codebook *book, oggpack_buffer *b);
 extern long vorbis_book_decodevs(codebook *book, double *a, oggpack_buffer *b,
 				 int step,int stagetype);
+extern long s_vorbis_book_decodevs(codebook *book, double *a, oggpack_buffer *b,
+				 int step,int stagetype);
 
 #endif
diff -ur vorbis-CLEAN/lib/codebook.c vorbis/lib/codebook.c
--- vorbis-CLEAN/lib/codebook.c	Fri Jul  7 03:37:00 2000
+++ vorbis/lib/codebook.c	Tue Aug 22 07:30:18 2000
@@ -268,7 +268,7 @@
 
 Residue0 encoding interleaves, uses multiple stages, and each stage
 peels of a specific amount of resolution from a lattice (thus we want
-to match by threshhold, not nearest match).  Residue doesn't *have* to
+to match by threshold, not nearest match).  Residue doesn't *have* to
 be encoded that way, but to change it, one will need to add more
 infrastructure on the encode side (decode side is specced and simpler) */
 
@@ -309,14 +309,24 @@
    Cascades may be additive or multiplicitive; this is not inherent in
    the codebook, but set in the code using the codebook.  Like
    interleaving, it's easiest to do it here.  
-   stage==0 -> declarative (set the value)
-   stage==1 -> additive
-   stage==2 -> multiplicitive */
+   addmul==-1 -> declarative (set the value)
+   addmul==0  -> additive
+   addmul==1  -> multiplicitive */
 
 /* returns the entry number or -1 on eof *************************************/
 long vorbis_book_decode(codebook *book, oggpack_buffer *b){
   long ptr=0;
   decode_aux *t=book->decode_tree;
+
+int lok = _oggpack_look(b, t->tabn);
+
+if (lok >= 0) {
+	ptr = t->tab[lok];
+	_oggpack_adv(b, t->tabl[lok]);
+	if (ptr <= 0)
+		return -ptr;
+}
+
   do{
     switch(_oggpack_read1(b)){
     case 0:
@@ -329,6 +339,7 @@
       return(-1);
     }
   }while(ptr>0);
+
   return(-ptr);
 }
 
@@ -337,22 +348,74 @@
 			  int step,int addmul){
   long entry=vorbis_book_decode(book,b);
   int i,o;
+  double *t;
   if(entry==-1)return(-1);
+  t = book->valuelist+entry*book->dim;
+  switch(addmul){
+  case -1:
+    for(i=0,o=0;i<book->dim-3;i+=4,o+=4*step) {
+      a[o]=t[i];
+      a[o+step]=t[i+1];
+      a[o+2*step]=t[i+2];
+      a[o+3*step]=t[i+3];
+    }
+    for(;i<book->dim;i++,o+=step)
+      a[o]=t[i];
+    break;
+  case 0:
+    for(i=0,o=0;i<book->dim-3;i+=4,o+=4*step) {
+      a[o]+=t[i];
+      a[o+step]+=t[i+1];
+      a[o+2*step]+=t[i+2];
+      a[o+3*step]+=t[i+3];
+    }
+    for(;i<book->dim;i++,o+=step)
+      a[o]+=t[i];
+    break;
+  case 1:
+    for(i=0,o=0;i<book->dim-3;i+=4,o+=4*step) {
+      a[o]*=t[i];
+      a[o+step]*=t[i+1];
+      a[o+2*step]*=t[i+2];
+      a[o+3*step]*=t[i+3];
+    }
+    for(;i<book->dim;i++,o+=step)
+      a[o]*=t[i];
+    break;
+  }
+  return(entry);
+}
+
+/* returns the entry number or -1 on eof *************************************/
+long s_vorbis_book_decodevs(codebook *book,double *a,oggpack_buffer *b,
+			  int step,int addmul){
+  long entry[step];
+  double *t[step];
+  int i,j,o;
+
+  for (i = 0; i < step; i++) {
+    entry[i]=vorbis_book_decode(book,b);
+    if(entry[i]==-1)return(-1);
+    t[i] = book->valuelist+entry[i]*book->dim;
+  }
   switch(addmul){
   case -1:
     for(i=0,o=0;i<book->dim;i++,o+=step)
-      a[o]=(book->valuelist+entry*book->dim)[i];
+      for (j=0;j<step;j++)
+	a[o+j]=t[j][i];
     break;
   case 0:
     for(i=0,o=0;i<book->dim;i++,o+=step)
-      a[o]+=(book->valuelist+entry*book->dim)[i];
+      for (j=0;j<step;j++)
+	a[o+j]+=t[j][i];
     break;
   case 1:
     for(i=0,o=0;i<book->dim;i++,o+=step)
-      a[o]*=(book->valuelist+entry*book->dim)[i];
+      for (j=0;j<step;j++)
+	a[o+j]*=t[j][i];
     break;
   }
-  return(entry);
+  return(entry[0]); /* return code is not valid, but not used either */
 }
 
 #ifdef _V_SELFTEST
diff -ur vorbis-CLEAN/lib/envelope.c vorbis/lib/envelope.c
--- vorbis-CLEAN/lib/envelope.c	Tue Aug 15 11:09:42 2000
+++ vorbis/lib/envelope.c	Thu Aug 24 20:05:06 2000
@@ -153,6 +153,7 @@
   /* we want to have a 'minimum bar' for energy, else we're just
      basing blocks on quantization noise that outweighs the signal
      itself (for low power signals) */
+#if 0
   {
     double min=ve->minenergy;
     for(i=0;i<n;i++){
@@ -160,6 +161,15 @@
       if(fabs(workB[i])<min)workB[i]=min;
     }
   }
+#else
+  {
+    double min=ve->minenergy;
+    for(i=0;i<n;i++){
+      if(workA[i]<min && workA[i]>-min)workA[i]=min;
+      if(workB[i]<min && workB[i]>-min)workB[i]=min;
+    }
+  }
+#endif
 
   /*_analysis_output("Afft",frameno,workA,n,0,0);
     _analysis_output("Bfft",frameno,workB,n,0,0);*/
diff -ur vorbis-CLEAN/lib/floor0.c vorbis/lib/floor0.c
--- vorbis-CLEAN/lib/floor0.c	Sat Aug 19 13:46:28 2000
+++ vorbis/lib/floor0.c	Tue Aug 22 06:23:53 2000
@@ -72,14 +72,14 @@
 
 /***********************************************/
 
-static void free_info(vorbis_info_floor *i){
+static void floor0_free_info(vorbis_info_floor *i){
   if(i){
     memset(i,0,sizeof(vorbis_info_floor0));
     free(i);
   }
 }
 
-static void free_look(vorbis_look_floor *i){
+static void floor0_free_look(vorbis_look_floor *i){
   vorbis_look_floor0 *look=(vorbis_look_floor0 *)i;
   if(i){
     if(look->linearmap)free(look->linearmap);
@@ -90,7 +90,7 @@
   }
 }
 
-static void pack (vorbis_info_floor *i,oggpack_buffer *opb){
+static void floor0_pack (vorbis_info_floor *i,oggpack_buffer *opb){
   vorbis_info_floor0 *info=(vorbis_info_floor0 *)i;
   int j;
   _oggpack_write(opb,info->order,8);
@@ -103,7 +103,7 @@
     _oggpack_write(opb,info->books[j],8);
 }
 
-static vorbis_info_floor *unpack (vorbis_info *vi,oggpack_buffer *opb){
+static vorbis_info_floor *floor0_unpack (vorbis_info *vi,oggpack_buffer *opb){
   int j;
   vorbis_info_floor0 *info=malloc(sizeof(vorbis_info_floor0));
   info->order=_oggpack_read(opb,8);
@@ -124,7 +124,7 @@
   }
   return(info);  
  err_out:
-  free_info(info);
+  floor0_free_info(info);
   return(NULL);
 }
 
@@ -136,8 +136,8 @@
    Note that the scale depends on the sampling rate as well as the
    linear block and mapping sizes */
 
-static vorbis_look_floor *look (vorbis_dsp_state *vd,vorbis_info_mode *mi,
-                              vorbis_info_floor *i){
+static vorbis_look_floor *floor0_look (vorbis_dsp_state *vd,
+			      vorbis_info_mode *mi, vorbis_info_floor *i){
   int j;
   double scale;
   vorbis_info        *vi=vd->vi;
@@ -259,8 +259,27 @@
 
 }
 
+void s_lsp_to_curve(double *curve,double *lsp,double amp,
+			  vorbis_look_floor0 *l,char *name,long frameno,
+			  double ampdB){
+  /* l->m+1 must be less than l->ln, but guard in case we get a bad stream */
+  double *lcurve=alloca(sizeof(double)*l->ln);
+  int i;
+
+  if(amp==0){
+    memset(curve,0,sizeof(double)*l->n);
+    return;
+  }
+  vorbis_lsp_to_curve(lcurve,l->ln,lsp,l->m,amp,l->lsp_look);
+
+  for (i = 0; i < l->ln; i++)
+    lcurve[i] = fromdB(lcurve[i]-ampdB);
+  for(i=0;i<l->n;i++)curve[i]=lcurve[l->linearmap[i]];
+
+}
+
 static long seq=0;
-static int forward(vorbis_block *vb,vorbis_look_floor *i,
+static int floor0_forward(vorbis_block *vb,vorbis_look_floor *i,
 		    double *in,double *out){
   long j;
   vorbis_look_floor0 *look=(vorbis_look_floor0 *)i;
@@ -388,8 +407,7 @@
 #endif
 
     /* take the coefficients back to a spectral envelope curve */
-    _lsp_to_curve(out,work,amp,look,"Ffloor",seq++);
-    for(j=0;j<look->n;j++)out[j]= fromdB(out[j]-info->ampdB);
+    s_lsp_to_curve(out,work,amp,look,"Ffloor",seq++,info->ampdB);
     return(1);
   }
 
@@ -398,7 +416,7 @@
   return(0);
 }
 
-static int inverse(vorbis_block *vb,vorbis_look_floor *i,double *out){
+static int floor0_inverse(vorbis_block *vb,vorbis_look_floor *i,double *out){
   vorbis_look_floor0 *look=(vorbis_look_floor0 *)i;
   vorbis_info_floor0 *info=look->vi;
   int j,k;
@@ -423,9 +441,8 @@
       }
       
       /* take the coefficients back to a spectral envelope curve */
-      _lsp_to_curve(out,out,amp,look,"",0);
+      s_lsp_to_curve(out,out,amp,look,"",0, info->ampdB);
       
-      for(j=0;j<look->n;j++)out[j]=fromdB(out[j]-info->ampdB);
       return(1);
     }
   }
@@ -437,7 +454,8 @@
 
 /* export hooks */
 vorbis_func_floor floor0_exportbundle={
-  &pack,&unpack,&look,&free_info,&free_look,&forward,&inverse
+  &floor0_pack,&floor0_unpack,&floor0_look,&floor0_free_info,&floor0_free_look,
+  &floor0_forward,&floor0_inverse
 };
 
 
diff -ur vorbis-CLEAN/lib/lsp.c vorbis/lib/lsp.c
--- vorbis-CLEAN/lib/lsp.c	Sat Aug 19 13:46:28 2000
+++ vorbis/lib/lsp.c	Thu Aug 24 18:02:08 2000
@@ -46,14 +46,13 @@
   for(i=0;i<m;i++)coslsp[i]=2*cos(lsp[i]);
 
   for(k=0;k<n;k++){
-    double p=.70710678118654752440;
-    double q=.70710678118654752440;
-    for(j=0;j<m;){
-      p*= *w-coslsp[j++];
-      q*= *w-coslsp[j++];
+    double p=.5;
+    double q=.5;
+    for(j=0;j<m;j+=2){
+      p*= w[k]-coslsp[j];
+      q*= w[k]-coslsp[j+1];
     }
-    curve[k]=amp/sqrt(p*p*(1.+ *w*.5)+q*q*(1.- *w*.5));
-    w++;
+    curve[k]=amp/sqrt(p*p*(2.+ w[k])+q*q*(2.- w[k]));
   }
 }
 
diff -ur vorbis-CLEAN/lib/mapping0.c vorbis/lib/mapping0.c
--- vorbis-CLEAN/lib/mapping0.c	Tue Aug 15 11:09:43 2000
+++ vorbis/lib/mapping0.c	Tue Aug 22 06:27:59 2000
@@ -56,14 +56,14 @@
 		     invalidate decay */
 } vorbis_look_mapping0;
 
-static void free_info(vorbis_info_mapping *i){
+static void mapping0_free_info(vorbis_info_mapping *i){
   if(i){
     memset(i,0,sizeof(vorbis_info_mapping0));
     free(i);
   }
 }
 
-static void free_look(vorbis_look_mapping *look){
+static void mapping0_free_look(vorbis_look_mapping *look){
   int i;
   vorbis_look_mapping0 *l=(vorbis_look_mapping0 *)look;
   if(l){
@@ -93,8 +93,8 @@
   }
 }
 
-static vorbis_look_mapping *look(vorbis_dsp_state *vd,vorbis_info_mode *vm,
-			  vorbis_info_mapping *m){
+static vorbis_look_mapping *mapping0_look(vorbis_dsp_state *vd,
+			  vorbis_info_mode *vm, vorbis_info_mapping *m){
   int i;
   vorbis_info *vi=vd->vi;
   vorbis_look_mapping0 *look=calloc(1,sizeof(vorbis_look_mapping0));
@@ -143,7 +143,7 @@
   return(look);
 }
 
-static void pack(vorbis_info *vi,vorbis_info_mapping *vm,oggpack_buffer *opb){
+static void mapping0_pack(vorbis_info *vi,vorbis_info_mapping *vm,oggpack_buffer *opb){
   int i;
   vorbis_info_mapping0 *info=(vorbis_info_mapping0 *)vm;
 
@@ -161,7 +161,7 @@
 }
 
 /* also responsible for range checking */
-static vorbis_info_mapping *unpack(vorbis_info *vi,oggpack_buffer *opb){
+static vorbis_info_mapping *mapping0_unpack(vorbis_info *vi,oggpack_buffer *opb){
   int i;
   vorbis_info_mapping0 *info=calloc(1,sizeof(vorbis_info_mapping0));
   memset(info,0,sizeof(vorbis_info_mapping0));
@@ -186,7 +186,7 @@
   return info;
 
  err_out:
-  free_info(info);
+  mapping0_free_info(info);
   return(NULL);
 }
 
@@ -203,7 +203,7 @@
 
 /* no time mapping implementation for now */
 static long seq=0;
-static int forward(vorbis_block *vb,vorbis_look_mapping *l){
+static int mapping0_forward(vorbis_block *vb,vorbis_look_mapping *l){
   vorbis_dsp_state     *vd=vb->vd;
   vorbis_info          *vi=vd->vi;
   vorbis_look_mapping0 *look=(vorbis_look_mapping0 *)l;
@@ -305,7 +305,7 @@
   return(0);
 }
 
-static int inverse(vorbis_block *vb,vorbis_look_mapping *l){
+static int mapping0_inverse(vorbis_block *vb,vorbis_look_mapping *l){
   vorbis_dsp_state     *vd=vb->vd;
   vorbis_info          *vi=vd->vi;
   vorbis_look_mapping0 *look=(vorbis_look_mapping0 *)l;
@@ -376,7 +376,8 @@
 
 /* export hooks */
 vorbis_func_mapping mapping0_exportbundle={
-  &pack,&unpack,&look,&free_info,&free_look,&forward,&inverse
+  &mapping0_pack,&mapping0_unpack,&mapping0_look,&mapping0_free_info,
+  &mapping0_free_look,&mapping0_forward,&mapping0_inverse
 };
 
 
diff -ur vorbis-CLEAN/lib/psy.c vorbis/lib/psy.c
--- vorbis-CLEAN/lib/psy.c	Tue Aug 15 13:53:18 2000
+++ vorbis/lib/psy.c	Thu Aug 24 21:26:41 2000
@@ -518,7 +518,7 @@
   memset(norm,0,n*sizeof(double));
 
   while(hi<n){
-    val=todB(f[i]*f[i])+400.;
+    val=todB_nn(f[i]*f[i])+400.;
     del=1./(i-lo);
     noise[lo]+=val*del;
     noise[i]-=val*del;
@@ -544,7 +544,7 @@
     long hilo=hi-lo;
 
     for(;i<n;i++){
-      val=todB(f[i]*f[i])+400.;
+      val=todB_nn(f[i]*f[i])+400.;
       del=1./(hii);
       noise[i]-=val*del;
       norm[i]-=del;
@@ -556,7 +556,7 @@
       norm[i]-=del;      
     }
     for(i=1,lo=n-ilo;lo<n;lo++,i++){
-      val=todB(f[n-i]*f[n-i])+400.;
+      val=todB_nn(f[n-i]*f[n-i])+400.;
       del=1./ilo;
       noise[lo]+=val*del;
       norm[lo]+=del;
diff -ur vorbis-CLEAN/lib/res0.c vorbis/lib/res0.c
--- vorbis-CLEAN/lib/res0.c	Tue Aug 15 11:09:43 2000
+++ vorbis/lib/res0.c	Thu Aug 24 23:08:13 2000
@@ -45,14 +45,14 @@
   int       **decodemap;
 } vorbis_look_residue0;
 
-void free_info(vorbis_info_residue *i){
+void res0_free_info(vorbis_info_residue *i){
   if(i){
-    memset(i,0,sizeof(vorbis_info_residue0));
+/*    memset(i,0,sizeof(vorbis_info_residue0));*/
     free(i);
   }
 }
 
-void free_look(vorbis_look_residue *i){
+void res0_free_look(vorbis_look_residue *i){
   int j;
   if(i){
     vorbis_look_residue0 *look=(vorbis_look_residue0 *)i;
@@ -67,7 +67,7 @@
   }
 }
 
-void pack(vorbis_info_residue *vr,oggpack_buffer *opb){
+void res0_pack(vorbis_info_residue *vr,oggpack_buffer *opb){
   vorbis_info_residue0 *info=(vorbis_info_residue0 *)vr;
   int j,acc=0;
   _oggpack_write(opb,info->begin,24);
@@ -87,7 +87,7 @@
 }
 
 /* vorbis_info is for range checking */
-vorbis_info_residue *unpack(vorbis_info *vi,oggpack_buffer *opb){
+vorbis_info_residue *res0_unpack(vorbis_info *vi,oggpack_buffer *opb){
   int j,acc=0;
   vorbis_info_residue0 *info=calloc(1,sizeof(vorbis_info_residue0));
 
@@ -112,11 +112,11 @@
 
   return(info);
  errout:
-  free_info(info);
+  res0_free_info(info);
   return(NULL);
 }
 
-vorbis_look_residue *look (vorbis_dsp_state *vd,vorbis_info_mode *vm,
+vorbis_look_residue *res0_look (vorbis_dsp_state *vd,vorbis_info_mode *vm,
 			  vorbis_info_residue *vr){
   vorbis_info_residue0 *info=(vorbis_info_residue0 *)vr;
   vorbis_look_residue0 *look=calloc(1,sizeof(vorbis_look_residue0));
@@ -235,9 +235,14 @@
   for(j=0;j<stages;j++){
     int dim=books[j]->dim;
     int step=n/dim;
+#if 0
     for(i=0;i<step;i++)
       if(vorbis_book_decodevs(books[j],work+i,opb,step,0)==-1)
 	return(-1);
+#else
+    if(s_vorbis_book_decodevs(books[j],work,opb,step,0)==-1)
+      return(-1);
+#endif
   }
   
   for(i=0;i<n;i++)
@@ -246,7 +251,7 @@
   return(0);
 }
 
-int forward(vorbis_block *vb,vorbis_look_residue *vl,
+int res0_forward(vorbis_block *vb,vorbis_look_residue *vl,
 	    double **in,int ch){
   long i,j,k,l;
   vorbis_look_residue0 *look=(vorbis_look_residue0 *)vl;
@@ -290,7 +295,7 @@
   /* we code the partition words for each channel, then the residual
      words for a partition per channel until we've written all the
      residual words for that partition word.  Then write the next
-     parition channel words... */
+     partition channel words... */
   
   for(i=info->begin,l=0;i<info->end;){
     /* first we encode a partition codeword for each channel */
@@ -324,7 +329,7 @@
 }
 
 /* a truncated packet here just means 'stop working'; it's not an error */
-int inverse(vorbis_block *vb,vorbis_look_residue *vl,double **in,int ch){
+int res0_inverse(vorbis_block *vb,vorbis_look_residue *vl,double **in,int ch){
   long i,j,k,l,transend=vb->pcmend/2;
   vorbis_look_residue0 *look=(vorbis_look_residue0 *)vl;
   vorbis_info_residue0 *info=look->info;
@@ -378,11 +383,11 @@
 }
 
 vorbis_func_residue residue0_exportbundle={
-  &pack,
-  &unpack,
-  &look,
-  &free_info,
-  &free_look,
-  &forward,
-  &inverse
+  &res0_pack,
+  &res0_unpack,
+  &res0_look,
+  &res0_free_info,
+  &res0_free_look,
+  &res0_forward,
+  &res0_inverse
 };
diff -ur vorbis-CLEAN/lib/scales.h vorbis/lib/scales.h
--- vorbis-CLEAN/lib/scales.h	Wed Jun 14 03:38:32 2000
+++ vorbis/lib/scales.h	Thu Aug 24 21:27:17 2000
@@ -24,6 +24,7 @@
 /* 20log10(x) */
 #define DYNAMIC_RANGE_dB 200.
 #define todB(x)   ((x)==0?-9.e40:log(fabs(x))*8.6858896)
+#define todB_nn(x)   ((x)==0?-9.e40:log(x)*8.6858896)
 #define fromdB(x) (exp((x)*.11512925))
 
 
diff -ur vorbis-CLEAN/lib/sharedbook.c vorbis/lib/sharedbook.c
--- vorbis-CLEAN/lib/sharedbook.c	Wed Jul 19 20:10:46 2000
+++ vorbis/lib/sharedbook.c	Thu Aug 24 23:10:28 2000
@@ -1,3 +1,4 @@
+#include <stdio.h>
 /********************************************************************
  *                                                                  *
  * THIS FILE IS PART OF THE Ogg Vorbis SOFTWARE CODEC SOURCE CODE.  *
@@ -125,6 +126,16 @@
     }    
   }
     
+#if 0
+for (i = 0; i < n; i++) {
+	fprintf(stderr, "%4d: ", i);
+	for (j = l[i] - 1; j >= 0; j--)
+		fprintf(stderr, "%d", (r[i] >> j) & 1);
+	fprintf(stderr, "\n");
+}
+fprintf(stderr, "\n");
+#endif
+
   /* bitreverse the words because our bitwise packer/unpacker is LSb
      endian */
   for(i=0;i<n;i++){
@@ -142,7 +153,7 @@
 /* build the decode helper tree from the codewords */
 decode_aux *_make_decode_tree(codebook *c){
   const static_codebook *s=c->c;
-  long top=0,i,j;
+  long top=0,i,j,n;
   decode_aux *t=malloc(sizeof(decode_aux));
   long *ptr0=t->ptr0=calloc(c->entries*2,sizeof(long));
   long *ptr1=t->ptr1=calloc(c->entries*2,sizeof(long));
@@ -173,6 +184,28 @@
     }
   }
   free(codelist);
+
+	t->tabn = _ilog(c->entries)-4; /* this is magic */
+	if(t->tabn<5)t->tabn=5;
+	n = 1<<t->tabn;
+	t->tab = malloc(n*sizeof(long));
+	t->tabl = malloc(n*sizeof(int));
+fprintf(stderr, "building %d book...\n", t->tabn);
+	for (i = 0; i < n; i++) {
+		long p = 0;
+		for (j = 0; j < t->tabn && (p > 0 || j == 0); j++) {
+			if (i & (1 << j))
+				p = ptr1[p];
+			else
+				p = ptr0[p];
+//fprintf(stderr, "p is %d\n", p);
+		}
+		/* now j == length, and p == -code */
+		t->tab[i] = p;
+		t->tabl[i] = j;
+fprintf(stderr, "book %02lx: l = %ld, p = %ld\n", i, j, p);
+	}
+
   return(t);
 }
 
diff -ur vorbis-CLEAN/lib/time0.c vorbis/lib/time0.c
--- vorbis-CLEAN/lib/time0.c	Fri Mar 10 14:21:18 2000
+++ vorbis/lib/time0.c	Tue Aug 22 06:26:55 2000
@@ -22,30 +22,31 @@
 #include "registry.h"
 #include "misc.h"
 
-static void pack (vorbis_info_time *i,oggpack_buffer *opb){
+static void time0_pack (vorbis_info_time *i,oggpack_buffer *opb){
 }
-static vorbis_info_time *unpack (vorbis_info *vi,oggpack_buffer *opb){
+static vorbis_info_time *time0_unpack (vorbis_info *vi,oggpack_buffer *opb){
   return "";
 
 }
-static vorbis_look_time *look (vorbis_dsp_state *vd,vorbis_info_mode *mi,
+static vorbis_look_time *time0_look (vorbis_dsp_state *vd,vorbis_info_mode *mi,
                               vorbis_info_time *i){
   return "";
 }
-static void free_info(vorbis_info_time *i){
+static void time0_free_info(vorbis_info_time *i){
 }
-static void free_look(vorbis_look_time *i){
+static void time0_free_look(vorbis_look_time *i){
 }
-static int forward(vorbis_block *vb,vorbis_look_time *i,
+static int time0_forward(vorbis_block *vb,vorbis_look_time *i,
 		    double *in,double *out){
   return(0);
 }
-static int inverse(vorbis_block *vb,vorbis_look_time *i,
+static int time0_inverse(vorbis_block *vb,vorbis_look_time *i,
 		    double *in,double *out){
   return(0);
 }
 
 /* export hooks */
 vorbis_func_time time0_exportbundle={
-  &pack,&unpack,&look,&free_info,&free_look,&forward,&inverse
+  &time0_pack,&time0_unpack,&time0_look,&time0_free_info,&time0_free_look,
+  &time0_forward,&time0_inverse
 };

